@@ -5,13 +5,12 @@
 #include <glib.h>
 
 #include "../kptraininglog.h"
+#include "../kppresetdata.h"
 #include "../kipina-i18n.h"
 #include "../kpsettings.h"
 #include "../kputil.h"
 
-#include "kpcalendarentryinfodialog.h"
 #include "kpnewsplitworkoutdialog.h"
-#include "kpnewworkoutdialog.h"
 #include "kpnewcommentdialog.h"
 #include "kpchartview.h"
 #include "kpviewmodel.h"
@@ -42,10 +41,14 @@ static void       kp_chart_view_get_dmy         (KPViewModel *view,
 static void       kp_chart_view_set_log         (KPViewModel *view,
                                                  KPTrainingLog *log);
 static void       kp_chart_view_unset_log       (KPViewModel *view);
+static void       kp_chart_view_activate        (KPViewModel *model);
+static void       kp_chart_view_deactivate      (KPViewModel *model);
 static void       kp_chart_view_set_view_type   (KPViewModel *view,
                                                  KPViewModelType type);
 static
 KPViewModelType   kp_chart_view_get_view_type   (KPViewModel *view);
+static gchar     *kp_chart_view_get_icon_name   (KPViewModel *view);
+
 static void       kp_chart_view_update          (KPChartView *cv);
 
 
@@ -189,11 +192,11 @@ static void
 kp_chart_view_init (KPChartView *cv)
 {
   KPChartViewPrivateData *p_data;
+  KPPresetDataItem *detail;
   GtkWidget *expander;
   GtkWidget *mode_combo;
   GtkWidget *param_combo;
   GtkWidget *hbox;
-  GSList *details;
   GSList *list;
   gchar *sport;
   gchar *param;
@@ -227,14 +230,11 @@ kp_chart_view_init (KPChartView *cv)
   for (i=0; i < (sizeof (const_params) / CONST_ENTRY_SIZE); i++)
     gtk_combo_box_append_text (GTK_COMBO_BOX (param_combo), _(const_params[i]));
     
-  list = kp_settings_get_list ("preset_detail");
-  details = list;
-
-  while (list) {
-    gtk_combo_box_append_text (GTK_COMBO_BOX (param_combo), list->data);
-    list = list->next;
+  list = kp_preset_data_get_items (KP_PRESET_DATA_DETAIL); 
+  for (;list; list = list->next) {
+    detail = KP_PRESET_DATA_ITEM (list->data);
+    gtk_combo_box_append_text (GTK_COMBO_BOX (param_combo), detail->name);
   }
-  kp_settings_list_free (details);
   
   gtk_combo_box_set_active (GTK_COMBO_BOX (param_combo), 0);
   gtk_box_pack_start (GTK_BOX (hbox), param_combo, FALSE, TRUE, 0);
@@ -265,6 +265,7 @@ kp_chart_view_init (KPChartView *cv)
                     G_CALLBACK (sport_combo_box_changed), cv);
   
   p_data->chart = kp_chart_new ();
+  kp_chart_set_n_groups (KP_CHART (p_data->chart), 1);
   gtk_box_pack_start (GTK_BOX (cv), p_data->chart, TRUE, TRUE, 0);
   gtk_widget_show (p_data->chart);
 
@@ -276,6 +277,19 @@ kp_chart_view_init (KPChartView *cv)
 
   g_string_assign (p_data->sport, (sport) ? sport : "");
   g_string_assign (p_data->param, (param) ? param : "");
+ 
+  kp_debug ("Initial sport: %s, initial param: %s\n", p_data->sport->str,
+            p_data->param->str);
+  
+  if (strcmp (p_data->param->str, "pace") == 0
+   || strcmp (p_data->param->str, "duration") == 0)
+    kp_chart_set_data_unit (KP_CHART (p_data->chart), KP_CHART_UNIT_MS);
+  else
+    kp_chart_set_data_unit (KP_CHART (p_data->chart), KP_CHART_UNIT_RAW);
+
+  kp_chart_view_update (cv);
+
+  g_signal_emit_by_name (param_combo, "changed");
 }
 
 static void
@@ -287,6 +301,9 @@ kp_chart_view_model_init (KPViewModelIface *iface)
   iface->unset_log = kp_chart_view_unset_log;
   iface->set_view_type = kp_chart_view_set_view_type;
   iface->get_view_type = kp_chart_view_get_view_type;
+  iface->get_icon_name = kp_chart_view_get_icon_name;
+  iface->activate = kp_chart_view_activate;
+  iface->deactivate = kp_chart_view_deactivate;
 }
 
 static void
@@ -385,12 +402,17 @@ kp_chart_view_get_view_type (KPViewModel *view)
   return p_data->type;
 }
 
+static gchar *
+kp_chart_view_get_icon_name (KPViewModel *view)
+{
+  return g_strdup ("chart.png");
+}
 
 static void
 kp_chart_view_set_log (KPViewModel *model, KPTrainingLog *log)
 {
   KPChartViewPrivateData *p_data;
-  KPSportEntry *entry;
+  KPSportEntryData *entry;
   GSList *list;
 
   kp_debug ("Setting log.");
@@ -406,8 +428,9 @@ kp_chart_view_set_log (KPViewModel *model, KPTrainingLog *log)
   list = kp_training_log_get_sports (log);
 
   while (list) {
-    entry = (KPSportEntry *) list->data;
-    gtk_combo_box_prepend_text (GTK_COMBO_BOX (p_data->sport_combo), entry->name);
+    entry = (KPSportEntryData *) list->data;
+    gtk_combo_box_prepend_text (GTK_COMBO_BOX (p_data->sport_combo),
+                                entry->name);
     list = list->next;
   }
   gtk_combo_box_set_active (GTK_COMBO_BOX (p_data->sport_combo), 0);
@@ -415,6 +438,8 @@ kp_chart_view_set_log (KPViewModel *model, KPTrainingLog *log)
   kp_chart_view_update (KP_CHART_VIEW (model));
 
   log_connect_signals (log, KP_CHART_VIEW (model));
+  
+  update_chart_data (KP_CHART_VIEW (model));
 }
 
 static void
@@ -441,7 +466,7 @@ kp_chart_view_update_dates (KPChartView *view)
 
   p_data = KP_CHART_VIEW_PRIVATE_DATA (view);
 
-  if (!p_data->date) 
+  if (!g_date_valid (p_data->date))
     return;
 
   /* For all time, the log is needed to figure out which are the first
@@ -455,12 +480,34 @@ kp_chart_view_update_dates (KPChartView *view)
 }
 
 
+static void
+kp_chart_view_activate (KPViewModel *model)
+{
+  KPChartViewPrivateData *p_data;
+  p_data = KP_CHART_VIEW_PRIVATE_DATA (model);
+
+  log_connect_signals (p_data->log, KP_CHART_VIEW (model));
+  kp_chart_view_update (KP_CHART_VIEW (model));
+}
+
+
+static void
+kp_chart_view_deactivate (KPViewModel *model)
+{
+  KPChartViewPrivateData *p_data;
+  p_data = KP_CHART_VIEW_PRIVATE_DATA (model);
+
+  log_disconnect_signals (p_data->log, KP_CHART_VIEW (model));
+}
+
+
 static void
 log_changed (KPTrainingLog *log, KPChartView *cv)
 {
   kp_chart_view_update (cv);
 }
 
+
 static void
 log_sport_list_changed (KPTrainingLog *log, gboolean added, const gchar *sport,
                         KPCalendarView *cv)
@@ -878,11 +925,15 @@ update_chart_data (KPChartView *view)
 
   if (!KP_IS_TRAINING_LOG (p_data->log))
     return;
+
+  kp_debug ("This goes fine!");
+
+  g_return_if_fail (KP_IS_CHART (p_data->chart));
   
   n_groups = kp_chart_get_n_groups (KP_CHART (p_data->chart));
 
   memset (&p_data->graph_data[0][0], 0, sizeof (gdouble)*ITEMS_MAX);
-  
+ 
   for (i=0; i < n_groups; i++) {
 
     sport = (p_data->sport->len > 0) ? p_data->sport->str : NULL;
@@ -927,7 +978,9 @@ update_chart_data (KPChartView *view)
     }
   }
   if (data) {
+    kp_chart_set_group_title (KP_CHART (p_data->chart), 0, p_data->param->str);
     kp_chart_set_n_items (KP_CHART (p_data->chart), n_items);
+    kp_chart_set_title (KP_CHART (p_data->chart), "");
     kp_chart_set (KP_CHART (p_data->chart), p_data->graph_data); 
   }
 }
