@@ -11,10 +11,10 @@
 #define NODE(d) ((GNode*)(d))
 
 /* GObject stuff */
-static void         kp_log_store_init            (KPLogStore * pkg_tree);
-static void         kp_log_store_class_init      (KPLogStoreClass * klass);
-static void         kp_log_store_tree_model_init (GtkTreeModelIface * iface);
-static void         kp_log_store_finalize        (GObject * object);
+static void         kp_log_store_init            (KPLogStore *pkg_tree);
+static void         kp_log_store_class_init      (KPLogStoreClass *klass);
+static void         kp_log_store_tree_model_init (GtkTreeModelIface *iface);
+static void         kp_log_store_finalize        (GObject *object);
 
 /* GtkTreeModel stuff */
 static GtkTreeModelFlags 
@@ -135,7 +135,7 @@ kp_log_store_class_init (KPLogStoreClass * klass)
 }
 
 static void
-kp_log_store_tree_model_init (GtkTreeModelIface * iface)
+kp_log_store_tree_model_init (GtkTreeModelIface *iface)
 {
   iface->get_flags = kp_log_store_get_flags;
   iface->get_n_columns = kp_log_store_get_n_columns;
@@ -152,22 +152,26 @@ kp_log_store_tree_model_init (GtkTreeModelIface * iface)
 }
 
 static void
-kp_log_store_init (KPLogStore * model)
+kp_log_store_init (KPLogStore *model)
 {
   GtkTreePath *path;
   GtkTreeIter iter;
 
-  g_assert (KP_LOG_STORE_N_COLUMNS == 1);
+  g_assert (KP_LOG_STORE_N_COLUMNS == 2);
 
-  model->column_types[0] = G_TYPE_STRING;	/* KP_LOG_STORE_COL_RECORD    */
+  model->type = 0;
+  
+  model->column_types[KP_LOG_STORE_COL_ICON] = GDK_TYPE_PIXBUF;
+  model->column_types[KP_LOG_STORE_COL_RECORD] = G_TYPE_STRING;
   model->n_columns = KP_LOG_STORE_N_COLUMNS;
   model->root = new_node (model, 0, 0, 0);
+  model->log = NULL;
+
   RECORD (model->root->data)->record = g_strdup (_("Log"));
 
-  do {
+  while (0 == model->stamp) {
     model->stamp = g_random_int ();
   }
-  while (0 == model->stamp);
 
   path = gtk_tree_path_new ();
   gtk_tree_path_append_index (path, 0);
@@ -177,10 +181,12 @@ kp_log_store_init (KPLogStore * model)
 }
 
 static void
-kp_log_store_finalize (GObject * object)
+kp_log_store_finalize (GObject *object)
 {
   GObjectClass *parent_class;
 
+  KP_LOG_STORE (object)->log = NULL;
+  
   parent_class = g_type_class_peek_parent (G_OBJECT_GET_CLASS (object));
   parent_class->finalize (object);
 }
@@ -191,15 +197,38 @@ kp_log_store_new (void)
   return KP_LOG_STORE (g_object_new (KP_TYPE_LOG_STORE, NULL));
 }
 
-void
-kp_log_store_attach_log (KPLogStore *store, KPTrainingLog *log)
+static void
+log_destroyed (KPTrainingLog *log, KPLogStore *store)
 {
+  store->log = NULL;
+}
+
+
+/**
+ * kp_log_store_attach:
+ * @store: A #KPLogStore
+ * @log: A #KPTrainingLog
+ * @type: Show only this kind of calendar entries
+ *
+ * Attach the log to log store. 
+ */
+void
+kp_log_store_attach_log (KPLogStore *store, KPTrainingLog *log, guint type)
+{
+  store->type = type;
+  store->log = log;
+  
   kp_training_log_foreach (log, (GFunc) kp_log_store_append_entry, store);
+  
+  kp_debug ("Log attached.");
+  
+  g_signal_connect (G_OBJECT (store->log), "destroy",
+                    G_CALLBACK (log_destroyed), store);
 }
 
 
 static GtkTreeModelFlags
-kp_log_store_get_flags (GtkTreeModel * tree_model)
+kp_log_store_get_flags (GtkTreeModel *tree_model)
 {
   g_return_val_if_fail (KP_IS_LOG_STORE (tree_model), (GtkTreeModelFlags) 0);
 
@@ -271,7 +300,7 @@ kp_log_store_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter,
 }
 
 static GtkTreePath *
-kp_log_store_get_path (GtkTreeModel * tree_model, GtkTreeIter * iter)
+kp_log_store_get_path (GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
   KPLogStore *model;
   GtkTreePath *path;
@@ -298,29 +327,75 @@ kp_log_store_get_path (GtkTreeModel * tree_model, GtkTreeIter * iter)
   return path;
 }
 
-static void
-kp_log_store_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
-			gint column, GValue *value)
+static GdkPixbuf *
+get_icon_for_entry (KPLogStore *store, KPLogStoreRecord *rec)
 {
-  KPLogStoreRecord *record;
+  KPCalendarEntry *entry;
+  GdkPixbuf *pb;
+  gchar *filename;
+  gchar *path;
+  
+  g_return_val_if_fail (store->log != NULL, NULL);
+  
+  if (rec->calendar_entry) {
+    entry = kp_training_log_get_entry (store->log, rec->d, rec->m, rec->y,
+                                       rec->record);
+
+    if (!entry) {
+      g_warning ("Couldn't get entry from log: \"%s\"! Why?", rec->record);
+      return NULL;
+    }
+  
+    filename = kp_calendar_entry_get_icon_name (entry);
+  } else {
+    filename = g_strdup ("folder.png");
+  }
+  
+  path = g_strconcat (KIPINA_PIXMAP_DIR, "/", filename, NULL);
+  g_free (filename);
+
+  pb = gdk_pixbuf_new_from_file (path, NULL);
+  
+  g_free (path);
+  return pb;
+}
+
+static void
+kp_log_store_get_value (GtkTreeModel *model, GtkTreeIter *iter, gint column,
+                        GValue *value)
+{
+  KPLogStoreRecord *rec;
+  GdkPixbuf *pixbuf;
   gchar *str;
 
-  g_return_if_fail (KP_IS_LOG_STORE (tree_model));
+  g_return_if_fail (KP_IS_LOG_STORE (model));
   g_return_if_fail (iter != NULL);
-  g_return_if_fail (column < KP_LOG_STORE (tree_model)->n_columns);
+  g_return_if_fail (column < KP_LOG_STORE (model)->n_columns);
 
-  g_value_init (value, KP_LOG_STORE (tree_model)->column_types[column]);
+  g_value_init (value, KP_LOG_STORE (model)->column_types[column]);
+  
+  rec = NODE (iter->user_data)->data;
+  g_return_if_fail (rec != NULL);
 
-  record = NODE (iter->user_data)->data;
-  g_return_if_fail (record != NULL);
-  g_assert (0 == column);
-
-  if (record->calendar_entry)
-    g_value_set_string (value, record->record);
-  else {
-    str = g_strdup_printf ("%s (%u)", record->record, record->n_entries);
-    g_value_set_string (value, str);
-    g_free (str);
+  switch (column)
+  {
+    case KP_LOG_STORE_COL_ICON:
+        pixbuf = get_icon_for_entry (KP_LOG_STORE (model), rec);
+        g_value_set_object (value, pixbuf);
+      break;
+    
+    case KP_LOG_STORE_COL_RECORD:
+      if (rec->calendar_entry)
+        g_value_set_string (value, rec->record);
+      else {
+        str = g_strdup_printf ("%s (%u)", rec->record, rec->n_entries);
+        g_value_set_string (value, str);
+        g_free (str);
+      }
+      break;
+    default:
+      kp_debug ("Get value called with column=%d", column);
+      g_return_if_reached ();
   }
 }
 
@@ -369,6 +444,16 @@ kp_log_store_iter_children (GtkTreeModel *model, GtkTreeIter *iter,
 }
 
 
+KPLogStoreRecord *
+kp_log_store_get_record (KPLogStore *store, GtkTreeIter *iter)
+{
+  g_return_val_if_fail (KP_IS_LOG_STORE (store), NULL);
+  g_return_val_if_fail (iter != NULL, NULL);
+  
+  return RECORD (NODE (iter->user_data)->data);
+}
+
+
 KPLogStoreRecordType
 kp_log_store_get_iter_type (KPLogStore *store, GtkTreeIter *iter)
 {
@@ -392,7 +477,7 @@ kp_log_store_get_iter_type (KPLogStore *store, GtkTreeIter *iter)
     return KP_LOG_STORE_REC_MONTH;
   
   if (g_date_valid_dmy (rec->d, rec->m, rec->y)) {
-    day = g_strdup_printf ("%d", rec->d);
+    day = g_strdup_printf ("%d.", rec->d);
     
     if (strcmp (rec->record, day) == 0)
       type = KP_LOG_STORE_REC_DAY;
@@ -406,17 +491,10 @@ kp_log_store_get_iter_type (KPLogStore *store, GtkTreeIter *iter)
 }
 
 
-/*****************************************************************************
- *
- *  kp_log_store_iter_has_child: Returns TRUE or FALSE depending on whether
- *                              the row specified by 'iter' has any children.
- *                              We only have a list and thus no children.
- *
- *****************************************************************************/
 static gboolean
-kp_log_store_iter_has_child (GtkTreeModel * tree_model, GtkTreeIter * iter)
+kp_log_store_iter_has_child (GtkTreeModel *model, GtkTreeIter *iter)
 {
-  g_return_val_if_fail (KP_IS_LOG_STORE (tree_model), FALSE);
+  g_return_val_if_fail (KP_IS_LOG_STORE (model), FALSE);
   g_return_val_if_fail (iter != NULL, FALSE);
   g_return_val_if_fail (iter->user_data != NULL, FALSE);
 
@@ -425,7 +503,7 @@ kp_log_store_iter_has_child (GtkTreeModel * tree_model, GtkTreeIter * iter)
 
 
 static gint
-kp_log_store_iter_n_children (GtkTreeModel * model, GtkTreeIter * iter)
+kp_log_store_iter_n_children (GtkTreeModel *model, GtkTreeIter *iter)
 {
   GNode *node;
   guint n=0;
@@ -453,35 +531,35 @@ kp_log_store_iter_n_children (GtkTreeModel * model, GtkTreeIter * iter)
 }
 
 static gboolean
-kp_log_store_iter_nth_child (GtkTreeModel * tree_model,
-			     GtkTreeIter * iter, GtkTreeIter * parent, gint n)
+kp_log_store_iter_nth_child (GtkTreeModel *model, GtkTreeIter *iter,
+                             GtkTreeIter *parent, gint n)
 {
   GNode *node;
 
-  g_return_val_if_fail (KP_IS_LOG_STORE (tree_model), FALSE);
+  g_return_val_if_fail (KP_IS_LOG_STORE (model), FALSE);
   g_return_val_if_fail (iter != NULL, FALSE);
 
   if (parent == NULL)
-    node = KP_LOG_STORE (tree_model)->root;
+    node = KP_LOG_STORE (model)->root;
   else
     node = g_node_nth_child (NODE (parent->user_data), n);
 
   if (!node)
     return FALSE;
 
-  iter->stamp = KP_LOG_STORE (tree_model)->stamp;
+  iter->stamp = KP_LOG_STORE (model)->stamp;
   iter->user_data = node;
 
   return TRUE;
 }
 
 static gboolean
-kp_log_store_iter_parent (GtkTreeModel * tree_model, GtkTreeIter * iter,
-			  GtkTreeIter * child)
+kp_log_store_iter_parent (GtkTreeModel *model, GtkTreeIter *iter,
+			  GtkTreeIter *child)
 {
   GNode *node;
 
-  g_return_val_if_fail (KP_IS_LOG_STORE (tree_model), FALSE);
+  g_return_val_if_fail (KP_IS_LOG_STORE (model), FALSE);
   g_return_val_if_fail (child != NULL, FALSE);
   g_return_val_if_fail (child->user_data != NULL, FALSE);
   g_return_val_if_fail (iter != NULL, FALSE);
@@ -496,7 +574,7 @@ kp_log_store_iter_parent (GtkTreeModel * tree_model, GtkTreeIter * iter,
 
   node = node->parent;
 
-  iter->stamp = KP_LOG_STORE (tree_model)->stamp;
+  iter->stamp = KP_LOG_STORE (model)->stamp;
   iter->user_data = node;
 
   return TRUE;
@@ -538,7 +616,7 @@ kp_log_store_add_mark (KPLogStore *store, guint d, guint m, guint y,
 }
 
 void
-kp_log_store_remove_mark (KPLogStore * store, guint d, guint m, guint y,
+kp_log_store_remove_mark (KPLogStore *store, guint d, guint m, guint y,
                           const gchar *mark)
 {
   GtkTreeIter iter;
@@ -674,9 +752,13 @@ kp_log_store_get_date (KPLogStore *store, GtkTreeIter *iter, guint *d,
 static void
 kp_log_store_append_entry (KPCalendarEntry *entry, KPLogStore *store)
 {
+  const gchar *mark;
   GDate *date;
-  gchar *mark;
 
+  if (store->type != 0) 
+    if (G_OBJECT_TYPE  (entry) != store->type)
+      return;
+  
   date = kp_calendar_time_get_date (entry->datetime);
   mark = kp_calendar_entry_to_string (entry);
 
@@ -685,7 +767,6 @@ kp_log_store_append_entry (KPCalendarEntry *entry, KPLogStore *store)
                          g_date_get_month (date),
                          g_date_get_year (date),
                          mark);
-  g_free (mark);
 }
 
 /*
@@ -693,7 +774,7 @@ kp_log_store_append_entry (KPCalendarEntry *entry, KPLogStore *store)
  */
 
 static GNode *
-add_year (KPLogStore * store, GNode * root, guint y, gint * pos)
+add_year (KPLogStore *store, GNode *root, guint y, gint *pos)
 {
   GNode *node = new_node (store, 0, 0, y);
 
@@ -710,12 +791,12 @@ add_year (KPLogStore * store, GNode * root, guint y, gint * pos)
 }
 
 static GNode *
-add_month (KPLogStore * store, GNode * y_node, guint m, guint y, gint * pos)
+add_month (KPLogStore *store, GNode *y_node, guint m, guint y, gint *pos)
 {
   GNode *node = new_node (store, 0, m, y);
 
   node = g_node_append (y_node, node);
-  RECORD (node->data)->record = g_strdup_printf ("%u", m);
+  RECORD (node->data)->record = g_strdup (kp_get_month_name (m-1));
   
   if (pos)
     {
@@ -726,12 +807,12 @@ add_month (KPLogStore * store, GNode * y_node, guint m, guint y, gint * pos)
 }
 
 static GNode *
-add_day (KPLogStore * store, GNode * m_node, guint d, guint m, guint y, gint * pos)
+add_day (KPLogStore *store, GNode *m_node, guint d, guint m, guint y, gint *pos)
 {
   GNode *node = new_node (store, d, m, y);
 
   node = g_node_append (m_node, node);
-  RECORD (node->data)->record = g_strdup_printf ("%u", d);
+  RECORD (node->data)->record = g_strdup_printf ("%u.", d);
 
   if (pos)
     {
@@ -742,7 +823,7 @@ add_day (KPLogStore * store, GNode * m_node, guint d, guint m, guint y, gint * p
 }
 
 static GNode *
-node_exists (GNode * parent, KPLogStoreRecordType type, guint n, gint * pos)
+node_exists (GNode *parent, KPLogStoreRecordType type, guint n, gint *pos)
 {
   GNode *node;
   gint i = 0;
@@ -756,30 +837,27 @@ node_exists (GNode * parent, KPLogStoreRecordType type, guint n, gint * pos)
       switch (type)
 	{
 	case KP_LOG_STORE_REC_DAY:
-	  if (RECORD (node->data)->d == n)
-	    {
-	      if (pos)
-		*pos = i;
-	      return node;
-	    }
+	  if (RECORD (node->data)->d == n) {
+	    if (pos)
+	      *pos = i;
+            return node;
+          }
 	  break;
 
 	case KP_LOG_STORE_REC_MONTH:
-	  if (RECORD (node->data)->m == n)
-	    {
-	      if (pos)
-		*pos = i;
-	      return node;
-	    }
+	  if (RECORD (node->data)->m == n) {
+            if (pos)
+              *pos = i;
+            return node;
+          }
 	  break;
 
 	case KP_LOG_STORE_REC_YEAR:
-	  if (RECORD (node->data)->y == n)
-	    {
-	      if (pos)
-		*pos = i;
-	      return node;
-	    }
+	  if (RECORD (node->data)->y == n) {
+            if (pos)
+              *pos = i;
+            return node;
+          }
 	  break;
 
 	case KP_LOG_STORE_REC_ROOT:
@@ -795,7 +873,7 @@ node_exists (GNode * parent, KPLogStoreRecordType type, guint n, gint * pos)
 }
 
 static GNode *
-new_node (KPLogStore * store, guint d, guint m, guint y)
+new_node (KPLogStore *store, guint d, guint m, guint y)
 {
   KPLogStoreRecord *rec;
   GNode *node;
