@@ -2,6 +2,7 @@
 #define __KP_NEW_COMMENT_DIALOG_H
 
 #include <gtk/gtk.h>
+#include "kpaddeditdialog.h"
 #include "kpcalendarview.h"
 #include "../kptraininglog.h"
 #include "../kpcomment.h"
@@ -15,18 +16,18 @@
 
 typedef struct
 {
-  GtkDialog       parent;
+  KPAddEditDialog       parent;
 
-  gpointer        private_data;
+  gpointer              private_data;
 } KPNewCommentDialog;
 
 typedef struct
 {
-  GtkDialogClass  parent_class;
+  KPAddEditDialogClass  parent_class;
 } KPNewCommentDialogClass;
 
 GType           kp_new_comment_dialog_get_type (void);
 
-GtkWidget      *kp_new_comment_dialog_new (GDate *date, KPComment *comment);
+GtkWidget      *kp_new_comment_dialog_new (KPDate *date, KPComment *comment);
 
 #endif /* __KP_NEW_COMMENT_DIALOG_H */
