@@ -27,6 +27,6 @@ typedef struct
 GType           kp_new_split_workout_dialog_get_type    (void);
 
 GtkWidget      *kp_new_split_workout_dialog_new         (KPDate *date,
-                                                         KPTrainingLog *log);
+                                                         KPCalendarEntry *entry);
 
 #endif /* __KP_NEW_SPLIT_WORKOUT_DIALOG_H */
