@@ -1,4 +1,5 @@
 #include <string.h>
+#include <math.h>
 
 #include <glib-object.h>
 #include <glib.h>
@@ -7,6 +8,7 @@
 #include "kpstatsview.h"
 #include "kpguiutils.h"
 
+#include "../kpworkoutmodel.h"
 #include "../kipina-i18n.h"
 
 static void            kp_stats_view_class_init        (GObjectClass *klass,
@@ -22,7 +24,8 @@ static void            kp_stats_update_dates           (KPStatsView *view);
 static void            kp_stats_set_statistics         (KPStatsView *model);
 static void            kp_stats_set_distance           (KPStatsView *model);
 static void            kp_stats_set_time               (KPStatsView *model);
-
+static void            kp_stats_view_activate          (KPViewModel *model);
+static void            kp_stats_view_deactivate        (KPViewModel *model);
 static void            kp_stats_view_set_dmy           (KPViewModel *view,
                                                         guint d,
                                                         guint m,
@@ -37,6 +40,7 @@ static void            kp_stats_view_unset_log         (KPViewModel *view);
 static void            kp_stats_view_set_view_type     (KPViewModel *view,
                                                         KPViewModelType type);
 static KPViewModelType kp_stats_view_get_view_type     (KPViewModel *view);
+static gchar          *kp_stats_view_get_icon_name     (KPViewModel *view);
 
 static void            kp_stats_set_field              (KPStatsView *model,
                                                         guint field,
@@ -53,7 +57,7 @@ static void            kp_stats_set_field_percent      (KPStatsView *model,
                                                         gdouble value);
 static void            kp_stats_set_field_time         (KPStatsView *model,
                                                         guint field,
-                                                        gdouble sec);
+                                                        guint msec);
 static void            sport_menu_selection_changed    (GtkOptionMenu *men,
                                                         KPStatsView *view);
 
@@ -191,10 +195,13 @@ kp_stats_view_model_init (KPViewModelIface *iface)
   iface->unset_log = kp_stats_view_unset_log;
   iface->set_view_type = kp_stats_view_set_view_type;
   iface->get_view_type = kp_stats_view_get_view_type;
+  iface->get_icon_name = kp_stats_view_get_icon_name;
+  iface->activate = kp_stats_view_activate;
+  iface->deactivate = kp_stats_view_deactivate;
 }
 
 static void
-kp_stats_view_instance_init (GObject * object, gpointer data)
+kp_stats_view_instance_init (GObject *object, gpointer data)
 {
   KPStatsViewPrivateData *p_data;
   KPStatsView *view;
@@ -306,7 +313,7 @@ kp_stats_set_distance (KPStatsView *model)
   gdouble sum;
   GList *list;
   guint n_days; /* how many days are there? */
-  guint n_dist; /* how many workouts has distance field? */
+  guint n_dist; /* how many workouts have distance field? */
   guint n;      /* how many workouts are there? */
 
   p_data = KP_STATS_VIEW_PRIVATE_DATA (model);
@@ -317,7 +324,7 @@ kp_stats_set_distance (KPStatsView *model)
 
   while (list) {
     if (kp_stats_filter (model, KP_CALENDAR_ENTRY (list->data))) {
-      distance = kp_workout_get_distance (KP_WORKOUT (list->data));
+      distance = kp_workout_model_get_distance (KP_WORKOUT_MODEL (list->data));
     
       if (n == 0) {
         min = distance;
@@ -344,19 +351,19 @@ kp_stats_set_distance (KPStatsView *model)
   kp_stats_set_field_double (model, LABEL_DISTANCE_PER_DAY,
                              sum / (gdouble) n_days);
   kp_stats_set_field_double (model, LABEL_DISTANCE_AVG,
-                            (n > 0) ? sum / (gdouble) n : 0);
+                            (n > 0) ? sum / (gdouble) n : 0.0);
   kp_stats_set_field_percent (model, LABEL_DISTANCE_PERCENT,
-                            (n > 0) ? (gdouble) n_dist / (gdouble) n : 0);
+                            (n > 0) ? (gdouble) n_dist / (gdouble) n : 0.0);
 }
 
 static void
 kp_stats_set_time (KPStatsView *model)
 {
   KPStatsViewPrivateData *p_data;
-  gdouble duration;
-  gdouble min;
-  gdouble max;
-  gdouble sum;
+  guint duration;
+  guint min;
+  guint max;
+  guint sum;
   GList *list;
   guint n_time;
   guint n_days;
@@ -365,20 +372,21 @@ kp_stats_set_time (KPStatsView *model)
   p_data = KP_STATS_VIEW_PRIVATE_DATA (model);
   list = p_data->entries;
 
-  sum = min = max = 0.0;
+  sum = min = max = 0;
   n_time = 0;
   n=0;
 
   while (list) {
     if (kp_stats_filter (model, KP_CALENDAR_ENTRY (list->data))) {
-      duration = kp_workout_get_duration_in_seconds (KP_WORKOUT (list->data));
+      duration = kp_workout_model_get_duration (KP_WORKOUT_MODEL (list->data));
+      kp_debug ("Got duration: %u", duration);
     
       if (n == 0) {
         min = duration;
         max = duration;
       }
       sum += duration;
-      n_time += (duration > 0.0);
+      n_time += (duration > 0);
       
       if (duration > 0 && duration < min) 
         min = duration;
@@ -396,10 +404,10 @@ kp_stats_set_time (KPStatsView *model)
   kp_stats_set_field_time (model, LABEL_TIME_MAX, max);
   kp_stats_set_field_time (model, LABEL_TIME_MIN, min);
   kp_stats_set_field_time (model, LABEL_TIME_PER_DAY, sum / (gdouble) n_days);
-  kp_stats_set_field_time (model, LABEL_TIME_AVG, (n > 0)
-                         ? sum / (gdouble) n : 0);
-  kp_stats_set_field_percent (model, LABEL_TIME_PERCENT, (n > 0)
-                           ? (gdouble) n_time / (gdouble) n : 0);
+  kp_stats_set_field_time (model, LABEL_TIME_AVG,
+                          (n > 0) ? floor ((gdouble) sum / (gdouble) n) : 0);
+  kp_stats_set_field_percent (model, LABEL_TIME_PERCENT,
+                          (n > 0) ? ((gdouble) n_time / (gdouble) n) : 0);
 
 }
 
@@ -528,6 +536,17 @@ kp_stats_view_get_dmy (KPViewModel *view, guint *d, guint *m, guint *y)
     *y = g_date_get_year (p_data->date);
 }
   
+static void
+kp_stats_view_activate (KPViewModel *model)
+{
+  /* nothing to do */
+}
+
+static void
+kp_stats_view_deactivate (KPViewModel *model)
+{
+  /* nothing to do */
+}
 
 static void
 kp_stats_view_set_log (KPViewModel *view, KPTrainingLog *log)
@@ -581,6 +600,11 @@ kp_stats_view_get_view_type (KPViewModel *view)
   return p_data->type;
 }
 
+static gchar *
+kp_stats_view_get_icon_name (KPViewModel *view)
+{
+  return g_strdup ("statistics.png");
+}
 
 static void
 sport_menu_selection_changed (GtkOptionMenu *menu, KPStatsView *view)
@@ -607,7 +631,7 @@ static gboolean
 kp_stats_filter (KPStatsView *view, KPCalendarEntry *entry)
 {
   KPStatsViewPrivateData *p_data;
-  gchar *sport;
+  const gchar *sport;
 
   if (!KP_IS_WORKOUT (entry))
     return FALSE;
@@ -677,16 +701,18 @@ kp_stats_set_field_double (KPStatsView *model, guint field, gdouble value)
 
 
 static void
-kp_stats_set_field_time (KPStatsView *model, guint field, gdouble sec)
+kp_stats_set_field_time (KPStatsView *model, guint field, guint msec)
 {
   guint h, m, s, t;
   gchar buf[32];
 
-  h = (guint) ((guint) sec / (60 * 60));
-  m = (guint) (((guint) sec - h * 60 * 60) / 60);
-  s = (guint) ((guint) sec - h * 60 * 60 - m * 60);
-  t = (guint) ((sec - h * 60 * 60 - m * 60 - s) * 1000);
+  h = (msec / (60 * 60 * 1000));
+  m = ((msec - h * 60 * 60 * 1000) / (60 * 1000));
+  s = (msec - h * 1000 * 60 * 60 - m * 60 * 1000) / 1000;
+  t = (msec - h * 1000 * 60 * 60 - m * 60 * 1000 - s * 1000);
 
+  kp_debug ("h: %u, m: %u, s: %u, t: %u", h, m, s, t);
+  
   g_snprintf (buf, sizeof (buf)-1,"%.2u:%.2u:%.2u.%.3u", h, m, s, t);
   kp_stats_set_field (model, field, buf, FALSE);
 }
