@@ -1,13 +1,15 @@
 #ifndef __KIPINA_INTL_H__
 #define __KIPINA_INTL_H__
 
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#ifndef LIBGNOME_H
 #ifdef ENABLE_NLS
-#include <libintl.h>
 #define _(String) gettext(String)
+#include <libintl.h>
 #ifdef gettext_noop
 #define N_(String) gettext_noop(String)
 #else
@@ -23,5 +25,6 @@
 #define bindtextdomain(Domain,Directory) (Domain) 
 #define bind_textdomain_codeset(Domain,Codeset) (Codeset) 
 #endif /* ENABLE_NLS */
+#endif /* LIBGNOME_H */
 
 #endif /* __KIPINA_INTL_H__ */
