@@ -27,15 +27,16 @@ typedef struct KPCommentClass_ {
   KPCalendarEntryClass parent_class;  
 } KPCommentClass;
 
-KPComment      *kp_comment_new (const gchar *title, const gchar *text);
-GType           kp_comment_get_type ();
-
-void            kp_comment_set_title (KPComment * comment, const gchar *title);
-void            kp_comment_set_text (KPComment * comment, const gchar *text);
-
-gchar          *kp_comment_get_title (KPComment * comment);
-gchar          *kp_comment_get_text (KPComment * comment);
-
-gchar          *kp_comment_get_formatted_date (KPComment * comment);
+GType           kp_comment_get_type             (void);
+KPComment      *kp_comment_new                  (const gchar *title, 
+                                                 const gchar *text);
+KPComment      *kp_comment_copy                 (KPComment *comment);
+void            kp_comment_set_title            (KPComment *comment, 
+                                                 const gchar *title);
+void            kp_comment_set_text             (KPComment *comment, 
+                                                 const gchar *text);
+gchar          *kp_comment_get_title            (KPComment *comment);
+gchar          *kp_comment_get_text             (KPComment *comment);
+gchar          *kp_comment_get_formatted_date   (KPComment *comment);
 
 #endif
