@@ -4,9 +4,6 @@
 #include <errno.h>
 #include <time.h>
 
-#include <libxslt/xsltutils.h>
-#include <libxslt/transform.h>
-#include <libxslt/xslt.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
@@ -14,10 +11,14 @@
 #include <glib.h>
 
 #include "kpcalendarentry.h"
+#include "kpsplitworkout.h"
+#include "kpworkoutmodel.h"
 #include "kptraininglog.h"
+#include "kppresetdata.h"
 #include "kpmarshalers.h"
 #include "kipina-i18n.h"
 #include "kpsettings.h"
+#include "kpresults.h"
 #include "kpcomment.h"
 #include "kpworkout.h"
 #include "kparray2d.h"
@@ -27,15 +28,16 @@ enum
 {
   SAVED_SIGNAL,
   CHANGED_SIGNAL,
-  ENTRY_ADDED_SIGNAL,
+  NEW_ENTRY_ADDED_SIGNAL,
   ENTRY_REMOVED_SIGNAL,
+  ENTRY_CHANGED_SIGNAL,
   SPORT_LIST_CHANGED_SIGNAL,
+  DESTROY_SIGNAL,
   LAST_SIGNAL
 };
 
 static guint      kp_training_log_signals[LAST_SIGNAL] = { 0 };
 
-static void       copy_css_file                     (const gchar *destdir);
 static
 KPTrainingLog    *kp_training_log_parse_from_xml    (const gchar *filename,
                                                      KPTrainingLog *log_,
@@ -50,6 +52,9 @@ static void       kp_training_log_class_init        (GObjectClass *klass,
 static void       kp_training_log_instance_init     (GObject *object,
                                                      gpointer data);
 static void       kp_training_log_instance_finalize (GObject *object);
+static void       kp_training_log_entry_changed     (KPCalendarEntry *entry,
+                                                     const gchar *old_string,
+                                                     KPTrainingLog *log);
 
 /* Handling sports */
 GSList           *find_sport                        (KPTrainingLog *log,
@@ -91,6 +96,7 @@ kp_training_log_xml_structured_error_func (void *ctx, xmlErrorPtr err)
   g_string_assign (last_error, err->message);
 }
 
+
 static void
 kp_training_log_class_init (GObjectClass *klass, gpointer data)
 {
@@ -106,7 +112,7 @@ kp_training_log_class_init (GObjectClass *klass, gpointer data)
                     G_STRUCT_OFFSET (KPTrainingLogClass, saved),
                     NULL,
                     NULL,
-                    g_cclosure_marshal_VOID__POINTER,
+                    g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);
 
@@ -120,22 +126,17 @@ kp_training_log_class_init (GObjectClass *klass, gpointer data)
                     g_cclosure_marshal_VOID__VOID,
                     G_TYPE_NONE,
                     0);
-  
-  kp_training_log_signals[ENTRY_ADDED_SIGNAL]
-    = g_signal_new ("entry-added",
+  kp_training_log_signals[NEW_ENTRY_ADDED_SIGNAL]
+    = g_signal_new ("new-entry-added",
                     G_OBJECT_CLASS_TYPE (object_class),
                     G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
-                    G_STRUCT_OFFSET (KPTrainingLogClass, entry_added),
+                    G_STRUCT_OFFSET (KPTrainingLogClass, new_entry_added),
                     NULL,
                     NULL,
-                    g_cclosure_user_marshal_VOID__UINT_UINT_UINT_STRING,
+                    g_cclosure_marshal_VOID__POINTER,
                     G_TYPE_NONE,
-                    4,
-                    G_TYPE_UINT,
-                    G_TYPE_UINT,
-                    G_TYPE_UINT,
-                    G_TYPE_STRING);
-
+                    1,
+                    G_TYPE_POINTER);
   kp_training_log_signals[ENTRY_REMOVED_SIGNAL]
     = g_signal_new ("entry-removed",
                     G_OBJECT_CLASS_TYPE (object_class),
@@ -151,6 +152,22 @@ kp_training_log_class_init (GObjectClass *klass, gpointer data)
                     G_TYPE_UINT,
                     G_TYPE_STRING);
   
+  kp_training_log_signals[ENTRY_CHANGED_SIGNAL]
+    = g_signal_new ("entry-changed",
+                    G_OBJECT_CLASS_TYPE (object_class),
+                    G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+                    G_STRUCT_OFFSET (KPTrainingLogClass, entry_changed),
+                    NULL,
+                    NULL,
+                    g_cclosure_user_marshal_VOID__UINT_UINT_UINT_STRING_STRING,
+                    G_TYPE_NONE,
+                    5,
+                    G_TYPE_UINT,
+                    G_TYPE_UINT,
+                    G_TYPE_UINT,
+                    G_TYPE_STRING,
+                    G_TYPE_STRING);
+  
   kp_training_log_signals[SPORT_LIST_CHANGED_SIGNAL]
     = g_signal_new ("sport-list-changed",
                     G_OBJECT_CLASS_TYPE (object_class),
@@ -164,6 +181,17 @@ kp_training_log_class_init (GObjectClass *klass, gpointer data)
                     G_TYPE_BOOLEAN,
                     G_TYPE_STRING);
 
+  kp_training_log_signals[DESTROY_SIGNAL]
+    = g_signal_new ("destroy",
+                    G_OBJECT_CLASS_TYPE (object_class),
+                    G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+                    G_STRUCT_OFFSET (KPTrainingLogClass, destroy),
+                    NULL,
+                    NULL,
+                    g_cclosure_marshal_VOID__VOID,
+                    G_TYPE_NONE,
+                    0);
+
   if (last_error == NULL)
     last_error = g_string_new (NULL);
 
@@ -176,10 +204,17 @@ kp_training_log_instance_init (GObject *object, gpointer data)
   KPTrainingLog *log;
 
   log = KP_TRAINING_LOG (object);
+
+  log->info_list = NULL;
   log->sports = NULL;
   log->year_list = NULL;
   log->n_workouts = 0;
   log->n_entries = 0;
+  log->id_table = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+  log->changed = FALSE;
+  log->written_to_disk = FALSE;
+  log->file = NULL;
 }
 
 
@@ -189,6 +224,7 @@ kp_training_log_instance_finalize (GObject *object)
   KPTrainingLog *log = KP_TRAINING_LOG (object);
   GObjectClass *parent_class;
 
+  g_signal_emit (object, kp_training_log_signals[DESTROY_SIGNAL], 0);
   kp_debug ("Finalized KPTrainingLog(%p) with %u entries.\n", log,
              kp_training_log_get_size (log));
  
@@ -200,6 +236,8 @@ kp_training_log_instance_finalize (GObject *object)
   parent_class->finalize (object);
 }
 
+
+
 /**
  * kp_training_log_new:
  * 
@@ -217,15 +255,78 @@ kp_training_log_new (void)
   return log;
 }
 
-KPTrainingLog *
-kp_training_log_new_from_file (const gchar *file)
+
+/**
+ * kp_training_log_get_filename:
+ * @log: A #KPTrainingLog
+ *
+ * Get filename that training log lives or is to be saved.
+ * 
+ * Returns: const string that is owned by log or NULL if the log
+ * is unnamed.
+ */
+G_CONST_RETURN gchar *
+kp_training_log_get_filename (KPTrainingLog *log)
 {
+  g_return_val_if_fail (KP_IS_TRAINING_LOG (log), NULL);
+  return log->file;
+}
+
+
+
+/**
+ * kp_training_log_set_filename:
+ * @log: A #KPTrainingLog
+ * @file: The filename
+ *
+ * Set the filename that the log will be saved when requested.
+ */
+void
+kp_training_log_set_filename (KPTrainingLog *log, const gchar *file)
+{
+  if (file == log->file)
+    return;
+  
+  if (log->file != NULL)
+    g_free (log->file);
+    
+  log->file = (file != NULL) ? g_strdup (file) : NULL;
+}
+
+
+/**
+ * kp_training_log_is_written_to_disk:
+ * @log: A #KPTrainingLog
+ * 
+ * This function checks if this log is loaded from a file or
+ * it has been saved to the disk while running the program.
+ * This is mainly intented for use by save routines.
+ * 
+ * Returns: TRUE if the log is loaded from the file or has
+ * been already saved to the file.
+ */
+gboolean
+kp_training_log_is_written_to_disk (KPTrainingLog *log)
+{
+  return log->written_to_disk;
+}
+
+
+KPTrainingLog *
+kp_training_log_new_from_file (const gchar *file, GError **err)
+{
+  GError *tmp_error = NULL;
   KPTrainingLog *log;
 
-  log = kp_training_log_parse_from_xml (file, NULL, NULL);
-  g_return_val_if_fail (KP_IS_TRAINING_LOG (log), NULL);
+  log = kp_training_log_parse_from_xml (file, NULL, &tmp_error);
+
+  if (!log) {
+    g_propagate_error (err, tmp_error);
+    return NULL;
+  }
 
   g_signal_emit (G_OBJECT (log), kp_training_log_signals[CHANGED_SIGNAL], 0);
+  log->changed = FALSE; 
   
   return log;
 }
@@ -244,11 +345,14 @@ gboolean
 kp_training_log_add_from_file (KPTrainingLog *log, const gchar *file,
                                GError **error)
 {
+  GError *tmp_error;
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
   g_return_val_if_fail (KP_IS_TRAINING_LOG (log), FALSE);
   g_return_val_if_fail (file != NULL, FALSE);
   
-  if (!kp_training_log_parse_from_xml (file, log, error)) {
+  tmp_error = NULL;
+  if (!kp_training_log_parse_from_xml (file, log, &tmp_error)) {
+    g_propagate_error (error, tmp_error);
     return FALSE;
   }
  
@@ -262,10 +366,11 @@ kp_training_log_error_quark (void)
   static GQuark quark = 0;
 
   if (!quark)
-    quark = g_quark_from_static_string ("training-log-error-quark");
+    quark = g_quark_from_static_string ("kp-training-log-error-quark");
   return quark;
 }
 
+
 void
 kp_training_log_destroy (KPTrainingLog * log)
 {
@@ -277,6 +382,15 @@ kp_training_log_destroy (KPTrainingLog * log)
   g_list_free (log->year_list);
 }
 
+
+gboolean
+kp_training_log_is_modified (KPTrainingLog *log)
+{
+  g_return_val_if_fail (KP_IS_TRAINING_LOG (log), FALSE);
+  return log->changed;
+}
+
+
 /**
  * kp_training_log_save:
  * @log: A #KPTrainingLog
@@ -304,153 +418,35 @@ kp_training_log_save (KPTrainingLog *log, const gchar *file, GError **error)
 
   val = xmlSaveFormatFile (file, doc, 1);
 
-  if (val > 0) {  
-    g_signal_emit (G_OBJECT (log), kp_training_log_signals[SAVED_SIGNAL], 0);
+  kp_debug ("xmlSaveFormatFile for '%s' returned %d (bytes).", file, val);
+  
+  if (val >= 0) {  
     ret_val = TRUE;
   }
   else {
     kp_debug ("Setting error!: %s", last_error->str);
    
-    if (error)
-      g_set_error (error,
-                   KP_TRAINING_LOG_ERROR,
-                   KP_TRAINING_LOG_SAVE_ERROR,
-                 _("The log (%s) couldn't be saved: %s!"), 
-                   file, last_error->str);
+    g_set_error (error,
+                 KP_TRAINING_LOG_ERROR,
+                 KP_TRAINING_LOG_SAVE_ERROR,
+               _("%s"), last_error->str);
 
     ret_val = FALSE;
   }
 
   xmlFreeDoc (doc);
 
+  if (ret_val) {
+    kp_debug ("Set filename: %s.", file);
+    kp_training_log_set_filename (log, file);
+    log->written_to_disk = TRUE;
+    log->changed = FALSE;
+    g_signal_emit (G_OBJECT (log), kp_training_log_signals[SAVED_SIGNAL], 0);
+  }
+  
   return ret_val;
 }
 
-/**
- * kp_training_log_create_html_stats:
- * @log: A #KPTrainingLog
- * @output_dir: Directory to output the HTML files
- * @error: location to store possible error
- *
- * Generate HTML statistics from data.
- * 
- * Returns: TRUE if successful and FALSE otherwise.
- */
-gboolean
-kp_training_log_create_html_stats (KPTrainingLog *log, const gchar *output_dir,
-                                   GError **error)
-{
-  xsltStylesheetPtr cur = NULL;
-  xmlDocPtr doc;
-  xmlDocPtr res = NULL;
-
-  gboolean retval = FALSE;
-
-  gchar *output_file;
-  gchar *stylesheet;
-  gchar *params[6];
-  gchar *language;
-
-  g_return_val_if_fail (KP_IS_TRAINING_LOG (log), FALSE);
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  doc = kp_training_log_to_xml (log);
-
-  language = kp_settings_get_str ("html_language");
-  
-  params[0] = "output_dir";
-  params[1] = g_strdup_printf ("'%s'", output_dir);
-  params[2] = "lang";
-  params[3] = (language) ? language : "en";
-  params[4] = NULL;
-  params[5] = NULL;
-
-  stylesheet = g_build_filename (KIPINA_DATA_DIR, "xsl", "kipina-report.xsl",
-                                 NULL);
-  output_file = g_build_filename (output_dir, "kipina-report.html", NULL);
- 
-  if (!g_file_test (output_dir, G_FILE_TEST_IS_DIR)) {
-    g_set_error (error,
-                 G_FILE_ERROR,
-                 g_file_error_from_errno (errno),
-              _("There is no directory \"%s\"!\nCan't create HTML statistics."
-                "\n\nCreate the directory and try again."),
-                 output_dir);
-    goto error;
-  }
-                 
-  cur = xsltParseStylesheetFile (stylesheet);
-  if (cur == NULL) {
-    g_set_error (error,
-                 KP_TRAINING_LOG_ERROR,
-                 KP_TRAINING_LOG_INTERNAL_ERROR,
-               _("Internal error happened! (Parsing stylesheet failed.)"
-                 "\n\nTHIS SHOULD NOT HAPPEN EVER!"));
-    goto error;
-  }
-  res = xsltApplyStylesheet (cur, doc, (const gchar **) params);
-  if (res == NULL) {
-    g_set_error (error,
-                 KP_TRAINING_LOG_ERROR,
-                 KP_TRAINING_LOG_INTERNAL_ERROR,
-               _("Internal error happened! (Applying stylesheet to xmlDoc"
-                 "failed.)\n\nTHIS SHOULD NOT HAPPEN EVER!"));
-    goto error;
-  }
-
-  retval = (xsltSaveResultToFilename(output_file, res, cur, 0) != -1);
-  if (retval) {
-    kp_debug ("Statistics saved to '%s'", output_file);
-    copy_css_file (output_dir);
-  } else {
-    g_set_error (error,
-                 KP_TRAINING_LOG_ERROR,
-                 KP_TRAINING_LOG_SAVE_HTML_ERROR,
-               _("HTML Statistics couldn't be saved to file \"%s\":"
-                 "Some error happened!"),
-                 output_file);
-    goto error;
-  }
-
-  retval = TRUE;
-error:
-  
-  xsltFreeStylesheet (cur);
-  xsltCleanupGlobals ();
-
-  if (res)
-    xmlFreeDoc (res);
-  if (doc)
-    xmlFreeDoc (doc);
-
-  g_free (params[1]);
-  g_free (stylesheet);
-  g_free (output_file);
-
-  return retval;
-}
-
-
-static void
-copy_css_file (const gchar * destdir)
-{
-  gint ret;
-  gchar *cmd;
-
-  cmd = g_strdup_printf ("cp %s/%s/%s %s", KIPINA_DATA_DIR,
-                         "xsl", "log.css", destdir);
-  ret = system (cmd);
-  if (ret != 0)
-    g_warning ("Couldn't copy log.css to output directory!");
-  g_free (cmd);
-
-  cmd = g_strdup_printf ("cp %s/%s/%s %s", KIPINA_DATA_DIR,
-                         "xsl", "horiz.png", destdir);
-  ret = system (cmd);
-  if (ret != 0)
-    g_warning ("Couldn't copy horiz.png to output directory!");
-  g_free (cmd);
-}
 
 /**
  * kp_training_log_add:
@@ -464,37 +460,29 @@ copy_css_file (const gchar * destdir)
 gboolean
 kp_training_log_add (KPTrainingLog *log, KPCalendarEntry *entry)
 {
-  KPSportEntry *s_entry;
+  KPSportEntryData *s_entry;
   GSList *list;
-  GDate *date;
+  KPDate date;
   TYear *ty;
-  gchar *str;
-  guint y;
-  guint m;
-  guint d;
+  const gchar *str;
   gint i;
 
   g_return_val_if_fail (log != NULL, FALSE);
   g_return_val_if_fail (entry != NULL, FALSE);
 
-  date = kp_calendar_time_get_date (entry->datetime);
-  g_return_val_if_fail (date != NULL, FALSE);
+  kp_calendar_entry_get_date (entry, &date);
 
-  y = g_date_get_year (date);
-  m = g_date_get_month (date);
-  d = g_date_get_day (date);
-
-  kp_debug ("Adding workout to the log, date: %u.%u.%u", d, m, y);
-  g_return_val_if_fail (g_date_valid_dmy (d, m, y), FALSE);
+  kp_debug ("Adding workout to log, date: %u.%u.%u", date.d, date.m, date.y);
+  g_return_val_if_fail (g_date_valid_dmy (date.d, date.m, date.y), FALSE);
   
-  ty = kp_training_log_get_year (log, y);
+  ty = kp_training_log_get_year (log, date.y);
 
   /* If there is no year in list, add it */
   if (ty == NULL) {
     ty = g_new (TYear, 1);
-    ty->year = y;
+    ty->year = date.y;
 
-    log->year_list = g_list_prepend (log->year_list, ty);
+    log->year_list = g_list_append (log->year_list, ty);
 
     for (i = 0; i < 12; i++)
       ty->workouts[i] = NULL;
@@ -503,20 +491,23 @@ kp_training_log_add (KPTrainingLog *log, KPCalendarEntry *entry)
     ty->n_entries = 0;
   }
 
-  g_return_val_if_fail (m > 0, FALSE);
-  g_return_val_if_fail (m <= 12, FALSE);
+  g_return_val_if_fail (date.m > 0, FALSE);
+  g_return_val_if_fail (date.m <= 12, FALSE);
 
+  if (g_list_find (ty->workouts[date.m - 1], entry)) {
+    g_warning ("Trying to add an entry to the log, but the entry is already in the log!");
+    g_return_val_if_reached (FALSE);
+  }
+  
   /* Insert entry to the month list */
-  ty->workouts[m - 1] = g_list_insert_sorted (ty->workouts[m - 1],
-                                              entry,
-                               (GCompareFunc) kp_calendar_entry_cmp);
+  ty->workouts[date.m - 1] = g_list_insert_sorted (ty->workouts[date.m - 1],
+                                                   entry,
+                                    (GCompareFunc) kp_calendar_entry_cmp);
 
-  str = kp_calendar_entry_to_string (entry);
+  log->changed = TRUE;
   g_signal_emit (G_OBJECT (log), kp_training_log_signals[CHANGED_SIGNAL], 0);
-  g_signal_emit (G_OBJECT (log), kp_training_log_signals[ENTRY_ADDED_SIGNAL],
-                 0, d, m, y, str);
-
-  g_free (str);
+  g_signal_emit (G_OBJECT (log), kp_training_log_signals[NEW_ENTRY_ADDED_SIGNAL],
+                 0, entry);
   
   if (KP_IS_WORKOUT (entry)) {
     ty->n_workouts++;
@@ -525,7 +516,7 @@ kp_training_log_add (KPTrainingLog *log, KPCalendarEntry *entry)
     str = kp_workout_get_sport (KP_WORKOUT (entry));
      
     if (!(list = find_sport (log, str))) {
-      s_entry = g_new (KPSportEntry, 1);
+      s_entry = g_new (KPSportEntryData, 1);
       s_entry->n = 1;
       s_entry->name = g_strdup (str);
         
@@ -533,11 +524,18 @@ kp_training_log_add (KPTrainingLog *log, KPCalendarEntry *entry)
       g_signal_emit (log, kp_training_log_signals[SPORT_LIST_CHANGED_SIGNAL], 0,
                      TRUE, str);
     } else {
-      ((KPSportEntry *) list->data)->n++;
+      ((KPSportEntryData *) list->data)->n++;
     }
   }
+  g_hash_table_insert (log->id_table, GINT_TO_POINTER (kp_calendar_entry_get_id (entry)), entry);
   ty->n_entries++;
   log->n_entries++;
+
+  kp_debug ("New entry added to the log with id: %u\n", 
+             kp_calendar_entry_get_id (entry));
+  
+  g_signal_connect (G_OBJECT (entry), "changed",
+                    G_CALLBACK (kp_training_log_entry_changed), log);
   
   return TRUE;
 }
@@ -546,7 +544,7 @@ kp_training_log_add (KPTrainingLog *log, KPCalendarEntry *entry)
 GSList *
 find_sport (KPTrainingLog *log, const gchar *sport)
 {
-  KPSportEntry *entry;
+  KPSportEntryData *entry;
   GSList *list;
   gchar *tmp1;
   gchar *tmp2;
@@ -555,7 +553,7 @@ find_sport (KPTrainingLog *log, const gchar *sport)
 
   while (list) {
      
-    entry = (KPSportEntry *) list->data;
+    entry = (KPSportEntryData *) list->data;
       
     tmp1 = g_utf8_collate_key (entry->name, -1);
     tmp2 = g_utf8_collate_key (sport, -1);
@@ -589,10 +587,10 @@ gboolean
 kp_training_log_remove (KPTrainingLog *log, KPCalendarEntry *entry)
 {
   GSList *list;
-  gchar *sport;
+  const gchar *sport;
   GDate *date;
   TYear *ty;
-  gchar *str;
+  const gchar *str;
   guint y;
   guint m;
   guint d;
@@ -619,14 +617,16 @@ kp_training_log_remove (KPTrainingLog *log, KPCalendarEntry *entry)
     sport = kp_workout_get_sport (KP_WORKOUT (entry));
     list = find_sport (log, sport);
    
-    if (list && ((KPSportEntry *) list->data)->n-- == 1) {
-      g_free (((KPSportEntry *) list->data)->name);
+    if (list && ((KPSportEntryData *) list->data)->n-- == 1) {
+      g_free (((KPSportEntryData *) list->data)->name);
       log->sports = g_slist_remove (log->sports, list->data);
  
       g_signal_emit (log, kp_training_log_signals[SPORT_LIST_CHANGED_SIGNAL], 0,
                      FALSE, sport);
     }
   }
+  g_hash_table_remove (log->id_table, GINT_TO_POINTER (kp_calendar_entry_get_id (entry)));
+  
   ty->n_entries--;
   log->n_entries--;
 
@@ -638,17 +638,28 @@ kp_training_log_remove (KPTrainingLog *log, KPCalendarEntry *entry)
   }
 
   kp_debug ("Removed: %d.%d.%d: %s\n", d, m, y, str);
+ 
+  g_object_set_data (G_OBJECT (log), "entry", entry);
   
   g_signal_emit (G_OBJECT (log), kp_training_log_signals[ENTRY_REMOVED_SIGNAL],
                  0, d, m, y, str);
 
   g_signal_emit (G_OBJECT (log), kp_training_log_signals[CHANGED_SIGNAL], 0);
-
+  log->changed = TRUE;
+  
   g_object_unref (entry);
 
   return TRUE;
 }
 
+
+static gboolean 
+always_true (gpointer key, gpointer val, gpointer data) 
+{ 
+  return TRUE; 
+}
+
+
 /**
  * kp_training_log_remove_all:
  * @log: A #KPTrainingLog
@@ -701,12 +712,14 @@ kp_training_log_remove_all (KPTrainingLog *log)
     y = y->next;
   } 
 
+  g_hash_table_foreach_remove (log->id_table, always_true, NULL);
   g_list_free (log->year_list);
-
+  
+  
   s = log->sports;
 
   while (s) {
-    sport = ((KPSportEntry *) s->data)->name;
+    sport = ((KPSportEntryData *) s->data)->name;
     g_signal_emit (log, kp_training_log_signals[SPORT_LIST_CHANGED_SIGNAL], 0,
                    FALSE, sport);
 
@@ -742,12 +755,101 @@ kp_training_log_get_sports (KPTrainingLog *log)
   return log->sports;
 }
 
+/**
+ * kp_training_log_get_info:
+ * @log: A #KPTrainingLog
+ *
+ * Returns info that is attached to the @log.
+ * 
+ * Returns: A list that MUST NOT BE MODIFIED!
+ */
+GSList *
+kp_training_log_get_info (KPTrainingLog *log)
+{
+  return log->info_list;
+}
 
+
+/**
+ * kp_training_log_get_info:
+ * @log: A #KPTrainingLog
+ * @field: field as a string
+ *
+ * Returns a string that is attached to field @field.
+ * 
+ * Returns: A string that MUST NOT BE MODIFIED OR FREED!
+ */
+G_CONST_RETURN gchar *
+kp_training_log_get_info_entry (KPTrainingLog *log, const gchar *field)
+{
+  GSList *node;
+  
+  for (node = log->info_list; node; node = node->next)
+    if (strcmp (field, ((KPInfoEntry *) node->data)->name) == 0)
+      return ((KPInfoEntry *) node->data)->data;
+
+  return NULL;
+}
+
+
+/**
+ * kp_training_log_add_info_entry:
+ * @log: A #KPTrainingLog
+ * @field: The field name
+ * @data: The data to attach
+ *
+ * Adds the information to the list of info entries.
+ * The data will be saved to the disk next time the
+ * log is saved.
+ */
+void
+kp_training_log_add_info_entry (KPTrainingLog *log, const gchar *field,
+                                const gchar *data)
+{
+  KPInfoEntry *info;
+  GSList *node;
+
+  g_return_if_fail (KP_IS_TRAINING_LOG (log));
+  g_return_if_fail (field != NULL);
+  g_return_if_fail (data != NULL);
+ 
+  /* Search the list if the entry is there already */
+  for (node = log->info_list; node; node = node->next) {
+    info = (KPInfoEntry *) node->data;
+    
+    if (strcmp (info->name, field) == 0) {
+      g_free (info->data);
+      info->data = g_strdup (data);
+      g_signal_emit (G_OBJECT (log), kp_training_log_signals[CHANGED_SIGNAL],
+                     0);
+      return;
+    }
+  }
+  /* Not found, so create a new structure */
+  info = g_new0 (KPInfoEntry, 1);
+  info->name = g_strdup (field);
+  info->data = g_strdup (data);
+
+  log->info_list = g_slist_prepend (log->info_list, info);
+  g_signal_emit (G_OBJECT (log), kp_training_log_signals[CHANGED_SIGNAL], 0);
+  log->changed = TRUE;
+}
+
+
+/**
+ * kp_training_log_get_earliest_date:
+ * @log: A #KPTrainingLog
+ *
+ * Get the date of the earliest workout or other entry 
+ * that is in the log.
+ * 
+ * Returns: A newly-allocated KPDate-structure that must
+ * be freed.
+ */
 KPDate *
 kp_training_log_get_earliest_date (KPTrainingLog *log)
 {
   GList *list;
-  guint d, m, y;
   GDate *gdate;
   KPDate *date;
   TYear *ty;
@@ -770,24 +872,13 @@ kp_training_log_get_earliest_date (KPTrainingLog *log)
   }
   kp_debug ("Found earliest year: %u", ty->year);
   
-  y = ty->year;
-
   for (i=0; i < 12; i++) {
     if (ty->workouts[i]) {
-
-      gdate = kp_calendar_time_get_date (kp_calendar_entry_get_date 
-        (KP_CALENDAR_ENTRY (ty->workouts[i]->data)));
-
-      g_return_val_if_fail (g_date_valid (gdate), NULL);
-
-      d = g_date_get_day (gdate);
-      m = g_date_get_month (gdate);
-      y = g_date_get_year (gdate);
-
-      date = kp_date_new_dmy (d, m, y);
+      
+      date = g_new0 (KPDate, 1);
+      kp_calendar_entry_get_date (KP_CALENDAR_ENTRY (ty->workouts[i]->data), 
+                                  date);
       g_return_val_if_fail (kp_date_valid (date), NULL);
-    
-      kp_debug ("First date: %u.%u.%u\n", d, m, y);
       
       return date;
     }
@@ -805,11 +896,21 @@ return_today:
   return date;
 }
 
+
+/**
+ * kp_training_log_get_latest_date:
+ * @log: A #KPTrainingLog
+ *
+ * Get the date of the latest workout or other entry 
+ * that is in the log.
+ * 
+ * Returns: A newly-allocated KPDate-structure that must
+ * be freed.
+ */
 KPDate *
 kp_training_log_get_latest_date (KPTrainingLog *log)
 {
   GList *list;
-  guint d, m, y;
   GDate *gdate;
   KPDate *date;
   TYear *ty;
@@ -830,25 +931,14 @@ kp_training_log_get_latest_date (KPTrainingLog *log)
 
     list = list->next;
   }
-  y = ty->year;
 
   for (i=11; i > 0; i--) {
     if ((list = g_list_last (ty->workouts[i]))) {
 
-      gdate = kp_calendar_time_get_date (kp_calendar_entry_get_date 
-        (KP_CALENDAR_ENTRY (list->data)));
-
-      g_return_val_if_fail (g_date_valid (gdate), NULL);
-
-      d = g_date_get_day (gdate);
-      m = g_date_get_month (gdate);
-      y = g_date_get_year (gdate);
-
-      date = kp_date_new_dmy (d, m, y);
+      date = g_new0 (KPDate, 1);
+      kp_calendar_entry_get_date (KP_CALENDAR_ENTRY (list->data), date);
       g_return_val_if_fail (kp_date_valid (date), NULL);
-
-      kp_debug ("Last date: %u.%u.%u\n", d, m, y);
-    
+      
       return date;
     }
   }
@@ -893,20 +983,17 @@ void
 kp_training_log_remove_mark (KPTrainingLog *log, guint d, guint m, guint y,
                              const gchar *mark)
 {
+  const gchar *m_str;
   GList *list;
-  gchar *m_str;
 
   list = kp_training_log_get_day (log, d, m, y);
   while (list) {
     m_str = kp_calendar_entry_to_string (KP_CALENDAR_ENTRY (list->data));
 
     if (strcmp (m_str, mark) == 0) {
-      g_free (m_str);
       kp_training_log_remove (log, KP_CALENDAR_ENTRY (list->data));
       return;
     }
-    else
-      g_free (m_str);
     
     list = list->next;
   }
@@ -996,6 +1083,25 @@ kp_training_log_remove_day (KPTrainingLog *log, guint d, guint m, guint y)
   }
 }
 
+static xmlNodePtr
+export_meta_data_xml (KPTrainingLog *log)
+{
+  GSList *list;
+  xmlNodePtr node;
+  xmlNodePtr info;
+
+  node = xmlNewNode (NULL, BAD_CAST ("meta"));
+
+  /* Add info fields */
+  for (list = log->info_list; list; list = list->next) {
+    info = xmlNewNode (NULL, BAD_CAST ("info"));
+    xmlSetProp (info, BAD_CAST ("field"), BAD_CAST (((KPInfoEntry *) list->data)->name));
+    xmlNodeSetContent (info, BAD_CAST (((KPInfoEntry *) list->data)->data));
+    xmlAddChild (node, info);
+  }
+  return node; 
+}
+
 
 /**
  * kp_training_log_to_xml:
@@ -1007,17 +1113,21 @@ kp_training_log_remove_day (KPTrainingLog *log, guint d, guint m, guint y)
  * Returns: #xmlDocPtr or NULL if some error occurs.
  */
 xmlDocPtr
-kp_training_log_to_xml (KPTrainingLog * log)
+kp_training_log_to_xml (KPTrainingLog *log)
 {
   xmlDocPtr doc;
   xmlNodePtr node;
+  xmlNodePtr meta;
 
   g_return_val_if_fail (KP_IS_TRAINING_LOG (log), NULL);
 
-  doc = xmlNewDoc ("1.0");
-  node = xmlNewChild ((xmlNodePtr) doc, NULL, "traininglog", NULL);
+  doc = xmlNewDoc (BAD_CAST ("1.0"));
+  node = xmlNewChild ((xmlNodePtr) doc, NULL, BAD_CAST ("traininglog"), NULL);
 
-  if (log->year_list)
+  meta = export_meta_data_xml (log);
+  xmlAddChild (node, meta);
+  
+  if (log->year_list) 
     g_list_foreach (log->year_list, kp_training_log_year_to_xml, node);
 
   return doc;
@@ -1029,7 +1139,6 @@ kp_training_log_year_to_xml (gpointer val, gpointer data)
   xmlDocPtr doc = (xmlDocPtr) data;
   GList *el;
   guint i;
-  guint n = 0;
   TYear *ty;
 
   ty = (TYear *) val;
@@ -1044,7 +1153,6 @@ kp_training_log_year_to_xml (gpointer val, gpointer data)
         
         xmlAddChild ((xmlNodePtr) doc,
                      kp_calendar_entry_to_xml (KP_CALENDAR_ENTRY (el->data)));
-        kp_debug ("Made XML node from entry %u.!", n++);
         el = el->next;
       }
     }
@@ -1068,18 +1176,16 @@ KPCalendarEntry *
 kp_training_log_get_entry (KPTrainingLog *log, guint d, guint m, guint y,
                            const gchar *mark)
 {
+  const gchar *m_str;
   GList *list;
-  gchar *m_str;
 
   list = kp_training_log_get_day (log, d, m, y);
   while (list) {
     m_str = kp_calendar_entry_to_string (KP_CALENDAR_ENTRY (list->data));
     
     if (strcmp (m_str, mark) == 0) {
-      g_free (m_str);
       return KP_CALENDAR_ENTRY (list->data);
     }
-    g_free (m_str);
     
     list = list->next;
   }
@@ -1281,8 +1387,9 @@ gdouble **
 kp_training_log_get_workout_params_between (KPTrainingLog *log,
                                             KPTrainingLogDataMode mode,
                                             const gchar *sport,
-                                            const gchar *param, GDate *start,
-                                            GDate *end, guint *days_between)
+                                            const gchar *param_name,
+                                            GDate *start, GDate *end,
+                                            guint *days_between)
 {
   KPWorkout *wo;
   GDate *date;
@@ -1293,6 +1400,7 @@ kp_training_log_get_workout_params_between (KPTrainingLog *log,
   guint idx;
 
   /* For avg */
+  const gchar *tmp_sport;
   gdouble *avg_sum;
   guint *avg_n;
 
@@ -1320,16 +1428,35 @@ kp_training_log_get_workout_params_between (KPTrainingLog *log,
    
     /* If this entry is a workout AND the sport is the same as wanted
      * OR NULL, then add param to the statistics */
+
+    if (wo == NULL)
+      goto loop_end;
     
-    if (wo && (!sport || strcasecmp (sport, kp_workout_get_sport (wo)) == 0)) {
+    tmp_sport = kp_workout_get_sport (wo);
+    g_return_val_if_fail (tmp_sport != NULL, NULL);
+    
+    if (sport == NULL || strcasecmp (sport, tmp_sport) == 0) {
+      
       date = kp_calendar_time_get_date (
           KP_CALENDAR_ENTRY (list->data)->datetime);
       g_assert (g_date_valid (date));
 
       idx = g_date_days_between (start, date);  
       g_assert (idx < days);
-  
-      item = kp_workout_get_param_double (KP_WORKOUT (list->data), param);
+ 
+      /* FIXME: quick hack to fix KPChartView */
+
+      item = 0.0;
+      if (strcmp (param_name, "pace") == 0)
+        item = kp_workout_model_get_pace (KP_WORKOUT_MODEL (wo));
+      if (strcmp (param_name, "duration") == 0)
+        item = kp_workout_model_get_duration (KP_WORKOUT_MODEL (wo));
+      if (strcmp (param_name, "distance") == 0)
+        item = kp_workout_model_get_distance (KP_WORKOUT_MODEL (wo));
+      else {
+        item = kp_param_list_get_double (KP_WORKOUT (list->data)->param_list, param_name);
+      }
+
       switch (mode)
       {
         case KP_TRAINING_LOG_DATA_SUM:      
@@ -1362,6 +1489,7 @@ kp_training_log_get_workout_params_between (KPTrainingLog *log,
           g_return_val_if_reached (NULL);
       }
     }
+loop_end:
     list = list->next;
   }
   if (days_between)
@@ -1380,9 +1508,9 @@ kp_training_log_get_workout_params_between (KPTrainingLog *log,
  * Returns: Two-dimensional gdouble-array of data 
  */
 gdouble **
-kp_training_log_get_workout_params_year (KPTrainingLog * log,
+kp_training_log_get_workout_params_year (KPTrainingLog *log,
                                          const gchar *param, guint year,
-                                         guint * days_between)
+                                         guint *days_between)
 {
   gdouble **ret_data;
   gdouble **data;
@@ -1392,13 +1520,12 @@ kp_training_log_get_workout_params_year (KPTrainingLog * log,
   guint m;
   guint i;
  
-  ret_data = (gdouble **) kp_array_2d_alloc_init0 (1, 12, sizeof (gdouble));
+  ret_data = kp_array_2dd_alloc_init0 (1, 12, sizeof (gdouble));
 
   s = g_date_new ();
   e = g_date_new ();
 
   for (m=1; m <= 12; m++) {
-    
     g_date_set_dmy (s, 1, m, year);
     g_date_set_dmy (e, kp_get_month_len (m, kp_leap (year)), m, year);
     
@@ -1408,7 +1535,7 @@ kp_training_log_get_workout_params_year (KPTrainingLog * log,
     for (i=0, ret_data[0][m-1]=0; i < days; i++)
       ret_data[0][m-1] += data[0][i];
     
-    kp_array_2d_free ((void **)data, 1, days);
+    kp_array_2dd_free (data, 1, days);
   }
   *days_between = 12;
   return ret_data;
@@ -1463,7 +1590,6 @@ kp_training_log_get_all_entries_between (KPTrainingLog *log, GDate *start,
       
       dlist = dlist->next;
     }
-    
     g_date_add_days (date, 1);
   }
   g_date_free (date);
@@ -1514,6 +1640,54 @@ kp_training_log_dump (KPTrainingLog *log)
 }
 
 
+static void
+log_add_info_data (KPTrainingLog *log, xmlNodePtr node)
+{
+  KPInfoEntry *info;
+
+  info = g_new0 (KPInfoEntry, 1);
+  info->name = (gchar *) xmlGetProp (node, BAD_CAST ("field"));
+  info->data = (gchar *) xmlNodeGetContent (node);
+  
+  if (info != NULL && info->name != NULL && info->data != NULL)
+    log->info_list = g_slist_append (log->info_list, info); 
+  else {
+    if (info->name == NULL)
+      g_warning ("<info> must have attribute 'field'!");
+    if (info->data == NULL)
+      g_warning ("<info> must have some content!");
+    
+    if (info->name)
+      g_free (info->name);
+    if (info->data)
+      g_free (info->data);
+    g_free (info);
+  }
+}
+
+static void
+parse_meta_data (KPTrainingLog *log, xmlNodePtr node)
+{
+  g_return_if_fail (node != NULL);
+  
+  node = node->children;
+
+  while (node) {
+    
+    /*** <presetlist> ***/
+    if (KP_TAG_MATCH (node, "presetlist")) {
+      (void) kp_preset_data_import_tree (node);
+    }
+
+    /*** <info> ***/
+    if (KP_TAG_MATCH (node, "info")) {
+      log_add_info_data (log, node);
+    }
+    
+    node = node->next;
+  }
+}
+ 
 
 static KPTrainingLog *
 kp_training_log_parse_from_xml (const gchar *filename, KPTrainingLog *log_,
@@ -1521,8 +1695,10 @@ kp_training_log_parse_from_xml (const gchar *filename, KPTrainingLog *log_,
 {
   KPCalendarEntry *entry;
   KPTrainingLog *log;
+  xmlNodePtr child;
   xmlNodePtr node;
   xmlDocPtr doc;
+  gboolean is_split;
 
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
@@ -1530,6 +1706,9 @@ kp_training_log_parse_from_xml (const gchar *filename, KPTrainingLog *log_,
     log = log_;
   else
     log = kp_training_log_new ();
+
+  log->file = g_strdup (filename);
+  log->written_to_disk = TRUE;
   
   doc = xmlParseFile (filename);
   if (!doc) 
@@ -1540,17 +1719,42 @@ kp_training_log_parse_from_xml (const gchar *filename, KPTrainingLog *log_,
     goto err;
 
   node = node->children;
-  
+ 
   while (node) {
+
+    /** <meta> **/
+    if (KP_TAG_MATCH (node, "meta")) {
+      parse_meta_data (log, node);
+    }
+    
     /** <workout> **/
-    if (KP_TAG_MATCH (node, "workout")) {
-      entry = KP_CALENDAR_ENTRY (kp_workout_new ());
-      
+    else if (KP_TAG_MATCH (node, "workout")) {
+      is_split = FALSE;
+      for (child = node->children; child; child = child->next) 
+        if (KP_TAG_MATCH (child, "split")) {
+          is_split = TRUE;
+          break;
+        }
+
+      if (is_split)
+        entry = KP_CALENDAR_ENTRY (kp_split_workout_new ());
+      else
+        entry = KP_CALENDAR_ENTRY (kp_workout_new ());
+
       if (!kp_calendar_entry_parse (KP_CALENDAR_ENTRY (entry), node))
         return NULL;
       g_return_val_if_fail (KP_IS_WORKOUT (entry), NULL);
       kp_training_log_add (log, entry);
     }
+   
+    else if (KP_TAG_MATCH (node, "results")) {
+      entry = KP_CALENDAR_ENTRY (kp_results_new (NULL, KP_RESULT_TYPE_TIME));
+      
+      if (!kp_calendar_entry_parse (KP_CALENDAR_ENTRY (entry), node))
+        return NULL;
+    
+      kp_training_log_add (log, entry);
+    }
     
     /** <comment> **/
     else if (KP_TAG_MATCH (node, "comment")) {
@@ -1566,9 +1770,8 @@ kp_training_log_parse_from_xml (const gchar *filename, KPTrainingLog *log_,
     else if (KP_TAG_MATCH (node, "text")) {
       /* DO NOTHING */
     }
-    else { 
-      g_warning ("Unknown tag in the log: %s\n", node->name);
-      goto err;
+    else {
+      /* The log may be made with newer version of Kipin */
     }
     node = node->next;
   }
@@ -1599,23 +1802,6 @@ err:
 }
 
 
-gdouble
-kp_training_log_get_week_total (KPTrainingLog *log,
-                                GDate *date, const gchar *param)
-{
-  gdouble sum = 0.0;
-  guint i;
-
-  g_return_val_if_fail (date != NULL, 0.0);
-  g_return_val_if_fail (g_date_valid (date), 0.0);
-
-  for (i = 0; i < 7; i++) {
-    sum += kp_training_log_get_param_day (log, date, param);
-    g_date_add_days (date, 1);
-  }
-  return sum;
-}
-
 /**
  * kp_training_log_get_param_day:
  * @log: A #KPTrainingLog.
@@ -1629,9 +1815,10 @@ kp_training_log_get_week_total (KPTrainingLog *log,
  */
 gdouble
 kp_training_log_get_param_day (KPTrainingLog *log, GDate *date,
-                               const gchar *param)
+                               const gchar *param_name)
 {
   KPCalendarEntry *entry;
+  KPParam *param = NULL;
   gdouble sum = 0.0;
   GList *list;
   GDate *wo_date;
@@ -1670,10 +1857,8 @@ kp_training_log_get_param_day (KPTrainingLog *log, GDate *date,
       return sum;
 
     if (KP_IS_WORKOUT (entry) && comp == 0) {
-      if (strcmp (param, "distance") == 0)
-        sum += kp_workout_get_distance (KP_WORKOUT (entry));
-      else
-        sum += kp_workout_get_param_double (KP_WORKOUT (entry), param);
+      param = kp_param_list_get_param (KP_WORKOUT (entry)->param_list, param_name);
+      sum += (param) ? kp_param_get_double (param) : 0;
     }
   }
 
@@ -1725,3 +1910,12 @@ kp_training_log_foreach (KPTrainingLog *log, GFunc func, gpointer data)
   }
 }
 
+static void
+kp_training_log_entry_changed (KPCalendarEntry *entry, const gchar *old_string,
+                               KPTrainingLog *log)
+{
+  g_signal_emit (G_OBJECT (log), kp_training_log_signals[CHANGED_SIGNAL], 0);
+  log->changed = TRUE;
+  kp_debug ("Entry in the log was changed!");
+}
+
