@@ -1,8 +1,12 @@
+#include <math.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <time.h>
+
+#include <gconf/gconf-client.h>
+#include <glib.h>
 
-#include <glade/glade.h>
 #include "kipina-i18n.h"
 #include "kpsettings.h"
 #include "kputil.h"
@@ -34,7 +38,6 @@ kp_date_set_time (KPDate *date, GTime time_)
 {
   GDate *gdate = g_date_new ();
   g_date_set_time (gdate, time_);
-  /*TODO: wtf?-) */
 
   date->d = g_date_get_day (gdate);
   date->m = g_date_get_month (gdate);
@@ -69,6 +72,37 @@ kp_date_new_dmy (guint d, guint m, guint y)
 }
 
 
+void
+kp_date_set_dmy (KPDate *date, guint d, guint m, guint y)
+{
+  g_return_if_fail (date != NULL);
+  date->d = d;
+  date->m = m;
+  date->y = y;
+}
+
+void
+kp_date_from_gdate (KPDate *date, GDate *gdate)
+{
+  date->d = g_date_get_day (gdate);
+  date->m = g_date_get_month (gdate);
+  date->y = g_date_get_year (gdate);
+}
+
+
+guint
+kp_date_get_iso8061_week_of_year (KPDate *date)
+{
+  guint wn;
+  guint ywn = date->y;
+
+  if (kp_week_of_year (&wn, &ywn, date->m, date->d))
+    return wn;
+  else 
+    return 0;
+}
+
+
 /**
  * kp_date_free:
  * @date: A #KPDate
@@ -83,6 +117,153 @@ kp_date_free (KPDate *date)
 }
 
 
+void
+kp_time_set_hms (KPTime *t, guint h, guint m, guint s)
+{
+  t->h = h;
+  t->m = m;
+  t->s = s;
+  t->initialized = TRUE;
+}
+
+/*
+ * kp_time_set_tod:
+ * @t: A #KPTime
+ * @tod: A #KPTod
+ *
+ * Set the time of the day as value of @t.
+ */
+void
+kp_time_set_tod (KPTime *t, KPTod tod)
+{
+  guint n;
+  
+  switch (tod) {
+    case KP_TOD_MORNING:
+      n = 10;
+      break;
+    case KP_TOD_NOON:
+      n = 12;
+      break;
+    case KP_TOD_AFTERNOON:
+      n = 16;
+      break;
+    case KP_TOD_EVENING:
+      n = 20;
+      break;
+    default:
+      g_return_if_reached ();
+  }
+
+  t->h = t->m = t->s = n;
+  t->initialized = TRUE;
+}
+
+/**
+ * kp_time_tod_to_string:
+ * @time: A #KPTime
+ *
+ * Turns tod to string presentation if the time
+ * is one of the tods recognized.
+ *
+ * Returns: Newly-allocated string or NULL
+ */
+gchar *
+kp_time_tod_to_string (KPTime *time) 
+{
+  if (time->initialized != TRUE)
+    return NULL;
+
+  if (!kp_time_is_tod (time, NULL)) {
+    g_print ("Time is not tod!");
+    return NULL;
+  }
+
+  switch (time->h) {
+    case 10:
+      return g_strdup (_("Morning"));
+    case 12:
+      return g_strdup (_("Noon"));
+    case 16:
+      return g_strdup (_("Afternoon"));
+    case 20:
+      return g_strdup (_("Evening"));
+  }
+  return NULL;
+}
+
+gboolean
+kp_time_is_tod (KPTime *time, KPTod *tod)
+{
+  g_return_val_if_fail (time != NULL, FALSE);
+
+  if (time->initialized != TRUE)
+    return FALSE;
+  
+  if (time->h != time->m || time->h != time->s)
+    return FALSE;
+
+  switch (time->h) {
+    case 10:
+      if (tod)
+        *tod = KP_TOD_MORNING;
+      break;
+    case 12:
+      if (tod)
+        *tod = KP_TOD_NOON;
+      break;
+    case 16:
+      if (tod)
+        *tod = KP_TOD_AFTERNOON;
+      break;
+    case 20:
+      if (tod)
+        *tod = KP_TOD_EVENING;
+      break;
+    default:
+      return FALSE;
+  }
+  return TRUE;
+}
+
+
+gboolean
+kp_date_dmy_to_locale_string (guint d, guint m, guint y, gchar *buf, 
+                              size_t len)
+{
+  KPDate date;
+
+  g_return_val_if_fail (g_date_valid_dmy (d, m, y), FALSE);
+ 
+  date.d = d;
+  date.m = m;
+  date.y = y;
+
+  return kp_date_to_locale_string (&date, buf, len);
+}
+
+
+gboolean
+kp_date_to_locale_string (KPDate *date, gchar *buf, size_t len)
+{
+  gchar *format;
+  struct tm t;
+  
+  g_return_val_if_fail (g_date_valid_dmy (date->d, date->m, date->y), FALSE);
+
+  t.tm_year = date->y - 1900;
+  t.tm_mon = date->m - 1;
+  t.tm_mday = date->d;
+  t.tm_hour = t.tm_min = t.tm_sec = 0;
+  
+  format = "%x";
+  strftime (buf, len-1, format, &t);
+
+  return TRUE;
+}
+
+
+
 gchar *
 kp_date_seconds_to_string (gdouble sec)
 {
@@ -153,25 +334,20 @@ kp_date_mseconds_to_std_string (guint32 msec)
  
   /* hours */
   h = (sec / 3600);
-  if (h)
-    g_string_append_printf (string, "%.2u:", h);
-
   m  = ((sec - h * 3600) / 60);
-  g_string_append_printf (string, "%.2u:", m);
-
   s = sec - h * 3600 - m * 60;
-  g_string_append_printf (string, "%.2u", s);
+  g_string_append_printf (string, "%.2u:%.2u:%.2u", h, m, s);
  
   /* (h * 3600 * 1000 + m * 60 * 1000 + s * 1000 < msec)*/
 
   ms = (msec % 1000) ? msec % 1000 : 0;
- 
+
   if (ms > 0)
     g_string_append_printf (string, ".%.3u", ms);
 
   ret = string->str;
   g_string_free (string, FALSE);
-  
+ 
   return ret;
 }
 
@@ -198,30 +374,27 @@ kp_is_valid_number (const gchar *possible_number)
  * Try to parse a string to a double.
  *
  * Returns: The double value if @possible_number is
- * valid and 0.0 otherwise.
+ * valid and -1.0 otherwise.
  */
 gdouble
 kp_number (const gchar *possible_number)
 {
-  guint i=0;
-  if (possible_number == NULL)
-    return -1.0;
-  
-  while (possible_number[i] != '\0') {
-    if (g_ascii_isdigit (possible_number[i]) != TRUE
-     && possible_number[i] != ','
-     && possible_number[i] != '.')
-    {
-      return -1.0;
-    }
+  gchar *end_ptr;
+  gdouble val;
 
-    i++;
-  }
-    
-  return g_ascii_strtod (possible_number, NULL);
+  g_return_val_if_fail (possible_number != NULL, -1.0);
+  
+  val = g_ascii_strtod (possible_number, &end_ptr);
+  if (possible_number != end_ptr && *end_ptr == '\0')
+    return val;
+  else
+    return -1.0;
 }
 
 /*
+ * kp_duration_str_to_ms:
+ * @time_str: String that contains the duration
+ * 
  * Test if time string is in one of the following formats:
  *
  * If len is 2, it's assumed to be: MM
@@ -234,14 +407,13 @@ kp_number (const gchar *possible_number)
  * we support also milliseconds. (That should be enough for
  * most people ;)
  *
- * Returns: the number of seconds (Can be decimal number as milliseconds
- * exist too).
+ * Returns: number of milliseconds
  */
 gint32
 kp_duration_str_to_ms (const gchar *time_str)
 {
-  gdouble val = 0.0;
-  guint coeff[6] = { 1, 10, 60, 600, 3600, 36000 };
+  gint val = 0;
+  guint coeff[7] = { 1, 10, 60, 600, 3600, 36000, 360000 };
   guint len;
   guint n;
   guint i;
@@ -249,28 +421,40 @@ kp_duration_str_to_ms (const gchar *time_str)
   gchar **str;
   gchar *tmp;
 
-  g_return_val_if_fail (time_str != NULL, -1.0);
+  g_return_val_if_fail (time_str != NULL, 0);
   
   len = strlen (time_str);
 
+  if (len == 0)
+    return 0;
+  
   /* MM */
-  if (len == 1 || len == 2)
-    return kp_number (time_str);
+  if (len == 1 || len == 2) {
+    num = kp_number (time_str);
+    if (num >= 0)
+      return num * 60 * 1000; /* Minutes to milliseconds */
+    else
+      return 0;
+  }
 
   tmp = g_strdup (time_str);
 
   /* .ddd */
-  if (len > 4 && time_str[len - 4] == '.') {
-    tmp[len - 4] = '\0';
-    val = kp_number (&tmp[len - 3]);
+  if (len == 12 && time_str[8] == '.') {
+    tmp[8] = '\0';
+    val = (gint) kp_number (&tmp[9]);
+    if (val < 0)
+      return 0;
+
     len = strlen (tmp);
   }
 
   /* HMM, HHMM, HMMSS, HHMMSS */
-  if ((num = kp_number (tmp)) >= 1 && len <= 6) {
+  num = kp_number (tmp);
+  if (num >= 0 && len <= 6) {
     for (i = len-1;;) {
       g_assert (i <= len-1);
-      val += coeff[len-1-i] * g_ascii_digit_value (tmp[i]) * 1000;
+      val += (gint) coeff[len-1-i] * g_ascii_digit_value (tmp[i]) * 1000;
 
       if (i==0)
         break;
@@ -282,29 +466,37 @@ kp_duration_str_to_ms (const gchar *time_str)
   /* HH:MM, HH:MM:SS */
   /* Split the string */
   str = g_strsplit (tmp, ":", 4);
-  for (n=1; str[n-1] != NULL; n++)
+  for (n=0; str[n] != NULL; n++) {
     if (n == 4) {
-      val = -1;
+      val = 0;
       goto error;
     }
-  
-  /* Count the sum */
-  for (i=0; i <= (n - 2) && str[i] != NULL;) {
-    if ((num = kp_number (str[i])) >= 0) {
-      if (num > 59) {
-        val = -1;
-        goto error;
-      }
-      val += coeff[n - i*2 + (n == 3)] * num * 1000;
+    if (kp_number (str[n]) < 0) {
+      val = 0;
+      goto error;
     }
-    if (i == (n - 2))
-      break;
-    i++;
   }
 
+  /* hh:mm:ss */
+  if (n == 3) {
+    val += ((guint) kp_number (str[0])) * 3600 * 1000;
+    val += ((guint) kp_number (str[1])) * 60 * 1000;
+    val += ((guint) kp_number (str[2])) * 1000;
+  }
+  /* mm:ss */
+  else if (n == 2) {
+    val += ((guint) kp_number (str[0])) * 60 * 1000;
+    val += ((guint) kp_number (str[1])) * 1000;
+  }
+  else {
+    val = 0;
+    goto error;
+  }
+ 
 error:
   g_strfreev (str);
   g_free (tmp);
+  
   return val;
 }
 
@@ -418,7 +610,7 @@ int main (void) {
 #endif
 
 gboolean
-kp_is_valid_time_str (const gchar * time_str)
+kp_is_valid_time_str (const gchar *time_str)
 {
   gchar **str;
   guint i;
@@ -490,8 +682,18 @@ kp_is_valid_date_str (const gchar * date_str)
   return FALSE;
 }
 
+/**
+ * kp_g_value_from_string:
+ * @string: A string that contains some value
+ *
+ * Tries to guess what kind of value that string represents
+ * and allocates space for the structure and initializes it with
+ * the correct value.
+ * 
+ * Returns: A pointer to GValue that must be freed.
+ */
 GValue *
-kp_g_value_from_string (const gchar * string)
+kp_g_value_from_string (const gchar *string)
 {
   GValue *value;
   gchar *err;
@@ -532,8 +734,17 @@ kp_g_value_from_string (const gchar * string)
   return value;
 }
 
+/**
+ * kp_g_value_to_string:
+ * @value: Pointer to GValue that is initialized
+ *
+ * Transform a GValue to a string presentation to show
+ * it to the user for example.
+ *
+ * Returns: A newly-allocated string that must be freed.
+ */
 gchar *
-kp_g_value_to_string (const GValue * value)
+kp_g_value_to_string (const GValue *value)
 {
   GType type = G_VALUE_TYPE (value);
 
@@ -594,6 +805,7 @@ month_names[12] = {
   N_("December"),
 };
 
+
 /**
  * kp_month_num_by_year_day_number:
  * @day_num: Number of day between 1-366
@@ -707,7 +919,7 @@ kp_calc_days (guint year, guint mm, guint dd)
 }
 
 gboolean
-kp_week_of_year (guint * week, guint * year, guint mm, guint dd)
+kp_week_of_year (guint *week, guint *year, guint mm, guint dd)
 {
   if (kp_check_date (*year, mm, dd)) {
     *week = kp_week_number (*year, mm, dd);
@@ -718,7 +930,7 @@ kp_week_of_year (guint * week, guint * year, guint mm, guint dd)
       (*year)++;
     }
     return TRUE;
-  }
+}
   return FALSE;
 }
 
@@ -736,13 +948,18 @@ void
 kp_print_debug_string (const gchar *file, const gchar *function, guint line,
     const gchar *message, ...)
 {
+  GConfClient *client;
   va_list args;
   gchar *format;
 
-  if (!kp_settings_get_bool ("debug"))
+#define KEY_DEBUG "/apps/kipina/preferences/debug"
+
+  client = gconf_client_get_default ();
+
+  if (!gconf_client_get_bool (client, KEY_DEBUG, NULL))
     return;
 
-  format = g_strdup_printf ("%s: %d, in %s(): %s\n", file, line, function,
+  format = g_strdup_printf ("%-20s: %-4d %s(): %s\n", file, line, function,
                             message);
   
   va_start (args, message);
@@ -750,6 +967,10 @@ kp_print_debug_string (const gchar *file, const gchar *function, guint line,
   va_end (args);
 
   g_free (format);
+
+  g_object_unref (client);
+  
+  fflush (stdout);
 }
 
 /**
@@ -758,15 +979,15 @@ kp_print_debug_string (const gchar *file, const gchar *function, guint line,
  * 
  * Get localized month name.
  *
- * Returns: Newly-allocated string that must be freed or
+ * Returns: Static string that must not be freed or 
  * NULL if @m is invalid.
  */
-gchar *
+G_CONST_RETURN gchar *
 kp_get_month_name (guint m)
 {
   g_return_val_if_fail (m < 12, NULL);
   
-  return g_strdup (month_names[m]);
+  return month_names[m];
 }
 
 
@@ -793,4 +1014,68 @@ kp_get_month_num (const gchar *m)
 
   return -1;
 }
-    
+
+
+void
+kp_autoscale_axis (gdouble *min, gdouble *max)
+{
+  gdouble pmin;                 /* proposal min */
+  gdouble pmax;                 /* proposal max */
+  gdouble rmin;                 /* real min */
+  gdouble rmax;                 /* real max */
+  gdouble tmp;
+  gint i;
+
+  rmin = *min;
+  rmax = *max;
+
+  g_return_if_fail (min != NULL);
+  g_return_if_fail (max != NULL);
+  g_return_if_fail (*min <= *max);
+
+  /* MINIMUMS */
+  if (rmin < 0) {
+    i = -1;
+    while (-pow (10, (gdouble) i++) > rmin);
+    /* nothing */
+    pmin = -pow (10, (gdouble)-- i);
+
+    while ((pmin / 2) < rmin && (pmin / rmin) > 1.0)
+      pmin /= 2;
+
+  }
+  else if (rmin == 0) {
+    pmin = 0;
+  }
+  else {
+    i = 0;
+    while (pow (10, ++i) <= rmin);
+    /* nothing */
+    pmin = pow (10, (gdouble)-- i);
+
+    while ((pmin * 2) <= rmin && (rmin / pmin) >= 1.0)
+      pmin *= 2;
+  }
+
+  if (*max == 0)
+    return;
+  
+  /* MAXIMUMS */
+  i = 0;
+  while (pow (10, i++) < rmax);
+  /* nothing */
+
+  pmax = pow (10, --i);
+
+  while ((pmax / 10) > rmax && (pmax / rmax) > 1.0)
+    pmax /= 10;
+
+  tmp = pmax / 10;
+  while (pmax - tmp >= rmax && (pmax / rmax) > 1.0)
+    pmax -= tmp;
+  /* Now pmax should be quite ok */
+
+  *min = pmin;
+  *max = pmax;
+}
+
